// -*- C++ -*-
/*!
 * @file  InUSBCamera.cpp
 * @brief USBCameraCaptureModule
 * @date $Date$
 *
 * $Id$
 */

#define HEADERSIZE	32

#include "InUSBCamera.h"
#include "MySampleGrabberCB.h"

extern CMySampleGrabberCB  *m_MySampleGrabberCB;

extern int FuncDshowMainStart( int , int , int , int );
extern int FuncDshowMainEnd( void );

long read_count = 0;

// Module specification
// <rtc-template block="module_spec">
static const char* inusbcamera_spec[] =
  {
    "implementation_id", "InUSBCamera",
    "type_name",         "InUSBCamera",
    "description",       "USBCameraCapture Component",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.frameWidth", "320",
    "conf.default.frameHeight", "240",
    "conf.default.propertyDisable", "0",
    // Widget
    "conf.__widget__.frameWidth", "text",
    "conf.__widget__.frameHeight", "text",
    "conf.__widget__.propertyDisable", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
InUSBCamera::InUSBCamera(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_outTimedImageDataOut("outTimedImageData", m_outTimedImageData),
    m_TimeServerServicePort("TimeServerService")

    // </rtc-template>
{
	  m_frameDepth = 3;
}

/*!
 * @brief destructor
 */
InUSBCamera::~InUSBCamera()
{
}



RTC::ReturnCode_t InUSBCamera::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("outTimedImageData", m_outTimedImageDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_TimeServerServicePort.registerConsumer("TimeServerInterface", "TimeServerInterface", m_stm);
  
  // Set CORBA Service Ports
  addPort(m_TimeServerServicePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("frameWidth", m_frameWidth, "320");
  bindParameter("frameHeight", m_frameHeight, "240");
  bindParameter("propertyDisable", m_propertyDisable, "0");
  

  // </rtc-template>
	timeBeginPeriod(1);
	return RTC::RTC_OK;
}


RTC::ReturnCode_t InUSBCamera::onFinalize()
{
	timeEndPeriod(1);
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t InUSBCamera::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InUSBCamera::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t InUSBCamera::onActivated(RTC::UniqueId ec_id)
{
	if(m_frameWidth > 640 || 0 >= m_frameWidth)
		m_frameWidth = 640;
	if(m_frameHeight > 480 || 0 >= m_frameHeight)
		m_frameHeight = 480;
	if(m_frameDepth > 3 || 0 >= m_frameDepth)
		m_frameDepth = 3;

	FuncDshowMainStart( m_frameWidth, m_frameHeight, m_frameDepth, m_propertyDisable );
	m_outTimedImageData.data.length(m_frameWidth * m_frameHeight * m_frameDepth + HEADERSIZE);
	m_framecounter = 0;
	printf("InUSBCamera Start\n");
	return RTC::RTC_OK;
}


RTC::ReturnCode_t InUSBCamera::onDeactivated(RTC::UniqueId ec_id)
{
	FuncDshowMainEnd();
	printf("InUSBCamera Stop\n");
	return RTC::RTC_OK;
}


RTC::ReturnCode_t InUSBCamera::onExecute(RTC::UniqueId ec_id)
{
	int i;
	SERVER_TM stm;
	bool ret=false;
	bool bfpResult = false;

	if( read_count != m_MySampleGrabberCB->record_readok_index ){
		read_count = m_MySampleGrabberCB->record_readok_index;
		if( (ret = Func_calc_timestamp( &stm )) == false ) {
			std::cout << "!!>>> Time Get Error <<<!!" << std::endl;
			return RTC::RTC_OK;
		}
		size_t size = m_frameWidth * m_frameHeight * m_frameDepth;

		memcpy_s(&m_outTimedImageData.data[0], 4, &stm.tm_hour, 4);
		memcpy_s(&m_outTimedImageData.data[4], 4, &stm.tm_min, 4);
		memcpy_s(&m_outTimedImageData.data[8], 4, &stm.tm_sec, 4);
		memcpy_s(&m_outTimedImageData.data[12], 4, &stm.tm_msec, 4);
		memcpy_s(&m_outTimedImageData.data[16], 4, &m_frameWidth, 4);
		memcpy_s(&m_outTimedImageData.data[20], 4, &m_frameHeight, 4);
		memcpy_s(&m_outTimedImageData.data[24], 4, &m_frameDepth, 4);
		memcpy_s(&m_outTimedImageData.data[28], 4, &m_framecounter, 4);

		for(i = 0; i < m_frameHeight; i++){
			memcpy_s(&(m_outTimedImageData.data[HEADERSIZE + i*m_frameWidth*m_frameDepth]), m_frameWidth * m_frameDepth,
				&(m_MySampleGrabberCB->eRINBUFF[read_count].imgdata[(m_frameHeight-i-1) * m_frameWidth*m_frameDepth]), m_frameWidth * m_frameDepth);
		}

		m_outTimedImageDataOut.write();
		m_framecounter++;
	}
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t InUSBCamera::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InUSBCamera::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InUSBCamera::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InUSBCamera::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InUSBCamera::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void InUSBCameraInit(RTC::Manager* manager)
  {
    coil::Properties profile(inusbcamera_spec);
    manager->registerFactory(profile,
                             RTC::Create<InUSBCamera>,
                             RTC::Delete<InUSBCamera>);
  }
  
};

//-----------------------------------------------------------------

bool InUSBCamera::Func_calc_timestamp( SERVER_TM *rstm )
{
	double tm1,tmb ;
	DWORD c_time,r_time;
	SERVER_TM stm;
	bool ret = m_stm->svr_getlocaltime(stm);

	if (false == ret) return false;

	c_time = timeGetTime();
	r_time = m_MySampleGrabberCB->eRINBUFF[read_count].timestanp;

    tm1 = stm.tm_hour * 3600000 + stm.tm_min *60000 + stm.tm_sec*1000 + stm.tm_msec;
    tmb = tm1 - (c_time-r_time);

    rstm->tm_hour  = (long)(tmb/3600000);
    rstm->tm_min   = (long)(tmb-rstm->tm_hour*3600000)/60000;
    rstm->tm_sec   = (long)(tmb-rstm->tm_hour*3600000-rstm->tm_min*60000)/1000;
    rstm->tm_msec  = (long)(tmb-(rstm->tm_hour*3600000+rstm->tm_min*60000+rstm->tm_sec*1000));

	return true;
}

//-----------------------------------------------------------------

bool InUSBCamera::Func_GetTimeStamp( SERVER_TM *rstm )
{
	DWORD c_time,r_time,s_time;
	SERVER_TM stm;
	bool ret = m_stm->svr_getlocaltime(stm);

	c_time = timeGetTime();
	r_time = m_MySampleGrabberCB->eRINBUFF[read_count].timestanp;
	if (true == ret)
	{
		rstm->tm_hour = stm.tm_hour;
		rstm->tm_min  = stm.tm_min;
		rstm->tm_sec  = stm.tm_sec;
		rstm->tm_msec = stm.tm_msec;

		s_time = c_time - r_time;
		printf("SUB time = %ld\n",s_time);
		rstm->tm_msec = stm.tm_msec - s_time;
		if( rstm->tm_msec < 0) {
			rstm->tm_sec = rstm->tm_sec - 1;
			rstm->tm_msec = 1000 - rstm->tm_msec;
			if(rstm->tm_sec < 0) {
				rstm->tm_min = rstm->tm_min - 1;
				rstm->tm_sec = 59;
				if(rstm->tm_min < 0) {
					rstm->tm_hour = rstm->tm_hour - 1;
					rstm->tm_min = 59;
					if(rstm->tm_hour < 0) {
						rstm->tm_hour = 23;
					}
				}
			}
		}

		return true;
	} else {
		rstm->tm_hour = 0;
		rstm->tm_min  = 0;
		rstm->tm_sec  = 0;
		rstm->tm_msec = 0;
		std::cout << "FALSE" << std::endl;

		return false;
	}
}